# $Id: MTime.pm,v 1.2 2000/11/14 23:20:07 dave Exp $
# Copyright (C) 2000 Paul Martin and Dave Woodcock. All rights reserved.

package MTime;

# {{{ preamble

use strict;
use vars qw
  (
   @ISA
   @EXPORT
   @EXPORT_OK
  );

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
@EXPORT = qw
  (
   &mtime
   &msleep
   &timed_exec
  );

bootstrap MTime;

# }}}
# {{{ timed_exec

# Execute the given code fragment, then wait for the rest of the pause
# (if any time is left).  Pause is specified in milliseconds.

sub timed_exec($$) {
  my ($pause, $code) = @_;

  my $start_time = mtime();
  $code->();
  my $time_taken = mtime() - $start_time;

  # Take care of clock rollover
  $time_taken += 86_400_000 if $time_taken < 0;

  # Sleep away the rest of the pause, if necessary
  my $sleep = $pause - $time_taken;
  msleep($sleep) if $sleep > 0;
  
  # Return number of milliseconds to sleep (possibly negative)
  $sleep;
}

# }}}

1;
__END__

# {{{ doco

# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

MTime - Perl extension for millisecond timing

=head1 SYNOPSIS

  use MTime;
  blah blah blah

=head1 DESCRIPTION

MTime supplies functions msleep($ms) - sleep for $ms milliseconds, and
mtime(), which returns the number of milliseconds since midnight GMT.

These are used in timed_exec($pause, $code), which executes the
coderef $code, and if $pause milliseconds have not already elapsed,
sleeps until they have. The function returns the number of
milliseconds of sleep that were needed (possibly negative).

=head1 AUTHOR

Dave Woodcock, david@anvil.com

=head1 SEE ALSO

perl(1).

=cut

# }}}
# {{{ Local Variables

# Local Variables:
# mode: cperl
# folded-file: t
# End:

# }}}
