# $Id: TiltPanel.pm,v 1.1 2000/11/22 20:02:28 dave Exp $
# Copyright (C) 2000 Paul Martin and Dave Woodcock. All rights reserved.

package TiltPanel;

# {{{ pragmas

use strict;
use strict;
# use AutoLoader 'AUTOLOAD';
use Tk;
use Getopt::Std;

# }}}
# {{{ new

sub new($%) {
  my $pkg = shift;
  my %args = @_;
  my $self = bless {}, $pkg;

  $self->read_config;
  $self->build_main;

  return $self;
}

# }}}
# {{{ build_main

sub build_main($) {
  my $self = shift;

  my $main = $self->{'main'} = MainWindow->new;
  $main->wm('grid');
  $main->pack('propagate' => 0);

  my $buttons = $self->{'buttons'} = {};
  my $oframe  = $self->{'oframe'}  = $main->Frame;
  $oframe->pack('expand' => 1, 'fill' => 'both');

  ################################################
  # Top Button Bar
  my $top_bar = $oframe->Frame('bd' => 1, 'relief' => 'raised');
  $top_bar->pack(side => 'top', fill => 'x');

  my $file_menu = $top_bar->Menubutton('text' => 'File', 'relief' => 'raised');
  foreach my $button ('print', 'quit') {
    my ($tag, $label) = ref($button) ? @$button : ($button, ucfirst $button);
    $buttons->{$tag} = $file_menu->command
      ('-label'      => $label, # inconsistency in PerlTk: can't be 'label'
       'command'     => [$self, $tag],
      );
  }
  $file_menu->pack(-side => 'left');

  foreach my $button ('home', 'back', 'forward', 'reload') {
    $self->add_button_name($button);
    $buttons->{$button} = $top_bar->Button
      ('text'    => ucfirst $button,
       'relief'  => 'raised',
       'command' => [$self, $button],
       'state'   => 'disabled',
      );
    $buttons->{$button}->pack(side => 'left');
  }

  ################################################
  # Parameters
  my $f = $oframe->Frame->pack(-side => 'top', fill => 'x');
  my $row = 0;
  foreach my $datum
    (['root_system', 'A3'],
     ['root_system', 'A3'],
     ['transient', []],
     ['periodic',  [0, 1, 2, 3]],
     ['frames',    200],
     ['pause',     100],
     ['step',      10],
     ['axis',      [1.9799371, 3.4292846, 62.145965]],
     ['astep',     0.01],
     ['ainit',     0.3],
     ['tstep',     0.01],
     ['pstep',     0.01],
     ['pinit',     0.5],
     ['rstep',     1.0],
     ['axes',      1],
     ['axes_mag',  20],
     ['ambient',   50],
     ['headlight', 50],
    ) {
      my $entry = $f->Entry('-relief' => 'sunken',  '-width' => 40);
      my $label = $f->Label(-text => $datum->[0], -anchor => 'e', -justify => 'right');
      Tk::grid($label, -row => $row, -column => 0, -sticky => 'e');
      Tk::grid($entry, -row => $row++, -column => 1,-sticky => 'ew');
      $f->gridRowconfigure(1,-weight => 1);
  }

  ################################################
  # Foot button bar
  my $foot_bar = $oframe->Frame('borderwidth' => 1, 'relief' => 'raised');
  $foot_bar->pack(side => 'bottom', fill => 'x');

  foreach my $button
    ('search', ['newbie', 'New'],
     'detail', 'select', 'insert', 'update', 'delete',
    ) {
      my ($tag, $text) = ref($button) ? @$button : ($button, ucfirst $button);
      $self->add_button_name($tag);
      $buttons->{$tag} = $foot_bar->Button
	('text'    => $text,
	 'relief'  => 'raised',
	 'command' => [$self, 'page_event', $tag],
	 'state'   => 'disabled',
	);
      $buttons->{$tag}->pack(side => 'left');
    }
  
  ################################################
  # Message area
  my $msg_area = $self->{'msg_area'} = $oframe->Label
    ('bd'     => 1,
     'relief' => 'sunken');
  $msg_area->pack('side' => 'bottom', 'fill' => 'x');

  ################################################
  # Scroll bars
  my $yframe = $oframe->Frame('bd' => 0);
  $yframe->pack('side' => 'right', 'fill' => 'y');
  $yframe->Frame
    ('bd'     => 0,
     'height' => 24
    )->pack('side' => 'bottom', 'fill' => 'x');

  my $yscrl = $self->{'yscrl'} = $yframe->Scrollbar('orient' => 'vert');
  $yscrl->pack('side' => 'top', 'fill' => 'y', 'expand' => 1);

  my $xscrl = $self->{'xscrl'} = $oframe->Scrollbar('orient' => 'horiz');
  $xscrl->pack('side' => 'bottom', 'fill' => 'x');

  $self->build_bindings;
}


# }}}
# {{{ loop

sub loop() {MainLoop}

# }}}
# {{{ add_button_name

sub add_button_name($$) {
  push @{$_[0]->{'button_names'}}, $_[1];
}

# }}}
# {{{ build_bindings

# Global key bindings

sub build_bindings($) {
  my $self = shift;
  my ($main, $yscrl) = ($self->{'main'}, $self->{'yscrl'});

  my @buttons =
    (
     'back'     => 'back_ev',
     'delete'   => 'delete_ev',
     'detail'   => 'detail_ev',
     'forward'  => 'forward_ev',
     'home'     => 'home_ev',
     'insert'   => 'insert_ev',
     'newbie'   => 'new_ev',
     'print'    => 'print_ev',
     'quit'     => 'quit_ev',
     'reconfig' => 'reconfig_ev',
     'reload'   => 'reload_ev',
     'search'   => 'search_ev',
     'select'   => 'select_ev',
     'update'   => 'update_ev',
    );

  while (@buttons) {
    $main->bind($self->config('Binding', pop @buttons),
		[$self => 'press', $self->{'buttons'}->{pop @buttons}],
	       );
  }

  $main->bind('<Prior>', [$yscrl => 'ScrlByPages', 'v', -1]);
  $main->bind('<Next>',  [$yscrl => 'ScrlByPages', 'v', 1]);
  $main->bind('<Home>',  [$yscrl => 'ScrlToPos', 0]);
  $main->bind('<End>',   [$yscrl => 'ScrlToPos', 1]);
}

# }}}
# {{{ build_text

sub build_text ($) {
  my $self = shift;
  my $oframe = $self->{'oframe'};

  my $text = $oframe->Text
    ('wrap'           => 'none',
     'xscrollcommand' => [$self->{'xscrl'} => 'set'],
     'yscrollcommand' => [$self->{'yscrl'} => 'set'],
    );

  $text->configure
    ('font'   => $self->config('Font',   'normal_font'),
     'cursor' => $self->config('Cursor', 'text_cursor'),
    );
  $text->tagConfigure('title', 	'font'	  => $self->config('Font', 'h1_font'));
  $text->tagConfigure('h1',    	'font'	  => $self->config('Font', 'h1_font'));
  $text->tagConfigure('h2',    	'font'	  => $self->config('Font', 'h2_font'));
  $text->tagConfigure('h3',    	'font'	  => $self->config('Font', 'h3_font'));
  $text->tagConfigure('bold',  	'font'	  => $self->config('Font', 'bold_font'));
  $text->tagConfigure('center', 'justify' => 'center');
  $text->tagConfigure
    ('link',
     'lmargin1'   => '1c',
     'lmargin2'   => '1c',
     'foreground' => $self->config('Colour', 'link_colour'),
    );

  $text->tagBind
    ('link',
     $self->config('Binding', 'follow_ev'),
     [$self => 'page_event', 'follow'],
    );

  $text->tagBind
    ('link',
     $self->config('Binding', 'choose_ev'),
     [$self => 'page_event', 'choose'],
    );

  $text->tagBind
    ('link',
     '<Enter>',
     [$text => 'configure', 'cursor', $self->config('Cursor', 'link_cursor')],
    );

  $text->tagBind
    ('link',
     '<Leave>',
     [$text => 'configure', 'cursor', $self->config('Cursor', 'text_cursor')],
    );

  $text->configure(-state => 'disabled');
  $text;
}

# }}}
# {{{ config

sub config($$$) {$_[0]->{'config'}->{$_[1] . '::' . $_[2]}}

# }}}
# {{{ read_config

sub read_config($) {
  my $self = shift;
  my $config = $self->{'config'} ||= {};

  foreach my $stem ('Font', 'Binding', 'Colour', 'Cursor', 'Misc') {
    my $file = join('/', $main::CONFIG, $stem . 'Choice');
    local *IN;
    open IN, "<$file" or die("Cannot open $file: $!\n");
      
    my (@keys, @vals);
    while (<IN>) {
      chomp;
      @keys = split(/,/)       if 1..1;
      @vals = split(/,/), last if 2..2;
    }

    close IN;

    for (my $i = 0; $i < @keys; ++$i) {
      # $main::log->print($stem. '::' . $keys[$i] . ' => ' . $vals[$i] . "\n", 3);
      $config->{$stem . '::' . $keys[$i]} = $vals[$i];
    }
  }
}

# }}}

# {{{ press

# Press the given button

sub press($$) {
  $_[1]->cget('state') eq 'normal' and $_[1]->cget('command')->Call;
}

# }}}
# {{{ message

sub message ($$) {
  my ($self, $msg) = @_;
  chomp($msg);
  $msg =~ s/\s+/ /o;
  $self->{'msg_area'}->configure('text' => $msg);
  $main::log->print("Message: $msg", 3);
  $self->{'timer'}->cancel if $self->{'timer'};
  $self->{'timer'} =
    $self->{'main'}->after(3000, sub {$self->clear_message});
}

# }}}
# {{{ clear_message

sub clear_message ($) {
  delete $_[0]->{'timer'};
  $_[0]->{'msg_area'}->configure(-text => '');
}

# }}}
# {{{ reconfig

sub reconfig($) {
  my $self = shift;
  $self->read_config;
  $self->configure_main;
  # $self->page->undraw;
  # $self->display;
}

# }}}
# {{{ print

sub print($) {
  my $self = shift;
  $self->{'slate'}->postscript('file' => "$main::DBVHOME/output.ps");
}

# }}}
# {{{ quit

sub quit($) {
  $_[0]->{'main'}->destroy;
  # $main::log->close;
  exit 0;
}

# }}}
# {{{ configure_main

sub configure_main($) {
  my $self = shift;
  $self->{'main'}->configure
    ('width'    => $self->config('Misc', 'width'),
     'height'   => $self->config('Misc', 'height'),
     'title'    => $self->config('Misc', 'title'),
     'iconname' => $self->config('Misc', 'iconname'),
    );
}

# }}}

1;

__END__

# {{{ Local Variables

# Local Variables:
# mode: cperl
# folded-file: t
# End:

# }}}
