# $Id: TiltingWalk.pm,v 1.1 2000/11/13 23:59:49 dave Exp $
# Copyright (C) 2000 Paul Martin and Dave Woodcock. All rights reserved.

package directedTiltingWalk;

# {{{ pragmas

use strict;
use Tilt;
use gvGeom;

# }}}
# {{{ new

# Args are:
#   $pkg:       package name
#   $ident:     canonical name of root system, e.g. 'A3'
#   $transient: array of ints specifying an initial transient 
#               sequence of Weyl reflections
#   $periodic:  array of ints specifying a periodic sequence
#               of Weyl reflections
#
# Sample values for $transient and $periodic:
#   A3: [], [0, 1, 2, 3]
#   A3: [], [0, 1, 2, 1, 3]
#   A3: [0, 1, 2], [3, 0, 2, 1]    # finite periodic
#   A3: [0, 1], [3, 0, 2, 1]       # infinite long
#   B2: [], [0, 2, 1, 2]
#   B3: [], [0, 2, 1, 3, 2, 1, 3]

sub new($$$$) {
  my ($pkg, $ident, $transient, $periodic) = @_;
  my $self = bless
    {
     'rootSystem'   => new rootSystem($ident),
     'transient'    => $transient,
     'periodic'     => $periodic,
     'in_transient' => scalar(@$transient),
     'is_periodic'  => scalar(@$periodic),
     'count'        => 0,
     'i'            => 0,
    }, $pkg;

  $self->{'s'} = $self->{'rootSystem'}->s;
  $self->{'w'} = $self->{'s'}->[0]->mult($self->{'s'}->[0]);
  $self;
}

# }}}
# {{{ next

sub next($) {
  my $self = shift;

  defined($self->{'w'}) or return undef;
  ++$self->{'count'};
  my $c = $self->{'w'}->tiltChar;
  my $k;

  if ($self->{'in_transient'}) {
    $k = $self->{'transient'}->[$self->{'i'}++];
    --$self->{'in_transient'} or $self->{'i'} = 0;

  } elsif ($self->{'is_periodic'}) {
    $k = $self->{'periodic'}->[$self->{'i'}++];
    $self->{'i'} < $self->{'is_periodic'} or $self->{'i'} = 0;

  } else {
    $self->{'w'} = undef;
    return $c;
  }

  # print STDERR "$k\n";
  $self->{'w'} = $self->{'s'}->[$k]->mult($self->{'w'});
  $c;
}

# }}}

package randomTiltingWalk;

# {{{ pragmas

use strict;
use Tilt;
use gvGeom;

# }}}
# {{{ new

# Args are:
#   $pkg:       package name
#   $ident:     canonical name of root system

sub new($$) {
  my ($pkg, $ident) = @_;
  my $self = bless
    {
     'rootSystem'   => new rootSystem($ident),
     'idx'          => 0,
     'count'        => 0,
    }, $pkg;

  $self->{'s'} = $self->{'rootSystem'}->s;
  $self->{'w'} = $self->{'s'}->[0]->mult($self->{'s'}->[0]);
  $self;
}

# }}}
# {{{ next

sub next($) {
  my $self = shift;
  ++$self->{'count'};
  my ($c, $x, $k) = ($self->{'w'}->tiltChar);

  do {
    $k = int(rand @{$self->{'s'}});
    $x = $self->{'w'}->reflectsDown($k) ? 0 : $self->{'s'}->[$k]->mult($self->{'w'});
  } until $x and $x->isDominant;
  $self->{'w'} = $x;

  $c;
}

# }}}

1;

__END__

# {{{ Local Variables

# Local Variables:
# mode: perl
# folded-file: t
# End:

# }}}

