WARNING;
// Copyright (C) 2000 Paul Martin and Dave Woodcock. All rights reserved.
// $Id: gvObj.m4,v 1.8 2000/11/13 06:16:39 dave Exp $
H_BEGIN(gvObj);

#include <iostream.h>
#include <fstream.h>
#include <unistd.h>
#include <sys/types.h>
#include "floatmat.h"
#include "misc.h"

// {{{ class colour

class colour {
public:
  float R;
  float G;
  float B;
  colour(float r = 0, float g = 0, float b = 0) : R(r), G(g), B(b) {};
  colour& operator= (const colour &c) {R = c.R; G = c.G; B = c.B; return *this;}
  static colour Colour[8];
  static const colour& Red;
  static const colour& Blue;
  static const colour& Green;
  static const colour& Yellow;
  static const colour& Cyan;
  static const colour& Purple;
  static const colour& Black;
  static const colour& White;
};

inline ostream& operator<< (ostream& s, const colour c) {
  return s << c.R << " " << c.G << " " << c.B << " ";
}

// }}}
// {{{ class gvObj

struct gvObj {
  char* name;    // object name
  ostream* conn; // ostream to print on

  gvObj(const char *n, ostream *s) : name(stringCopy(n)), conn(s) {
    if (!conn) BAD_ARG("gvObj::gvObj");
  }

  void setConn(ostream *s) {
    if(!(conn = s)) BAD_ARG("gvObj::setConn");
    conn = s;
  }

  gvObj::~gvObj() {
    if (name) delete [] name;
  }

  void send(const floatmat& m) {
    unsigned int n = m.width();
    static char* fnName = "gvObj::send(floatmat&):\n";
  
    if (n != m.height())
      BAD_ARG(fnName);

    switch (n) {
    case 2:
      *conn << m(0,0) << " " << m(0,1) << " 0 0 "
	    << m(1,0) << " " << m(1,1) << " 0 0 "
	    << "0 0 1 0 "
	    << "0 0 0 1 ";
      return;
    case 3:
      *conn << m(0,0) << " " << m(0,1) << " " << m(0,2) << " 0 "
	    << m(1,0) << " " << m(1,1) << " " << m(1,2) << " 0 "
	    << m(2,0) << " " << m(2,1) << " " << m(2,2) << " 0 "
	    << "0 0 0 1 ";
      return;
    case 4:
      *conn << m(0,0) << " " << m(0,1) << " " << m(0,2) << " " << m(0,3) << " "
	    << m(1,0) << " " << m(1,1) << " " << m(1,2) << " " << m(1,3) << " "
	    << m(2,0) << " " << m(2,1) << " " << m(2,2) << " " << m(2,3) << " "
	    << m(3,0) << " " << m(3,1) << " " << m(3,2) << " " << m(3,3) << " ";
      return;
    default:
      BAD_ARG(fnName);
    }
  }

  void send(const char* command) {
    *conn << "(" << command << " " << name
	  << ")\n";
  }
  void send(const char* command, bool b) {
    *conn << "(" << command << " " << name
	  << (b ? " yes" : " no") << ")\n";
  }
  void send(const char* command, const float f) {
    *conn << "(" << command << " " << name
	  << " " << f << ")\n";
  }
  void send(const char* command, const floatmat& m) {
    *conn << "(" << command << " " << name << " ";
    send(m);
    *conn << ")\n";
  }
  void send(const char* command, const colour& c) {
    *conn << "(" << command << " " << name
	  << " " << c.R << " " << c.G << " " << c.B << ")";
  }
  void send(const char* command, const char* arg1, const char* arg2 = "") {
    *conn << "(" << command << " " << name
	  << " " << arg1
	  << " " << arg2 << ")\n";
  }
  void sendPre(const char* command, const char* arg) {
    *conn << "(" << command << " " << arg
	  << " " << name << ")\n";
  }

  void del() { // Delete the object
    send("delete");
  }

  void xform(const floatmat& m) { // Concatenate m with the current transform
    send("xform", m);
  }

  void xformIncr(const floatmat& m) { // {{{ 
    // Apply continual motion: concatenate m with the current
    // transform every refresh // }}}
    send("xform-incr", m);
  }

  void xformSet(const floatmat& m) { // Overwrite the current transform
    send("xform-set", m);
  }
};

// }}}
// {{{ class gvGeom

class gvGeom : public gvObj {
public:
  void bboxColor(const colour& col) { // set colour of bbox
    send("bbox-color", col);
  }

  void bboxDraw(bool b) {
    send("bbox-draw", b);
  }

  void evert(bool b) { // Set the normal eversion state of GEOM-ID.
    send("evert", b);
  }

  void normalization(const char* s) {// {{{ 
    // Set the normalization status of GEOM-ID.
    // `none'
    //      suppresses all normalization.
    // `each'
    //      normalizes the object's bounding box to fit into the unit
    //      sphere, with the center of its bounding box translated
    //      to the origin.  The box is scaled such that its long diagonal,
    //      sqrt((xmax-xmin)^2 + (ymax-ymin)^2 + (zmax-zmin)^2), is 2.
    // `all'
    //      resembles `each', except when an object is changing
    //      (e.g. when its geometry is being changed by an external program).
    //      Then, `each' tightly fits the bounding box around the
    //      object whenever it changes and normalizes accordingly,
    //      while `all' normalizes the union of all variants of the object
    //      and normalizes accordingly.
    // `keep'
    //      leaves the current normalization transform unchanged
    //      when the object changes.  It may be useful to apply `each' or
    //      `all' normalization apply to the first version of a changing
    //      object to bring it in view, then switch to `keep'.// }}}
    send("normalization", s);
  }

  void set(const char* geomspec) { // {{{ 
    // Specify the geometry for GEOM-ID.  GEOMETRY is a string
    // giving an OOGL geometry specification.  If no object
    // called GEOM-ID exists, it is created; in this case the
    // GEOMETRY argument is optional, and if omitted, the new
    // object GEOM-ID is given an empty geometry.// }}}
    send("geometry", geomspec);
  }

  gvGeom(const char *name, ostream* s) : gvObj(name, s) {
    set("{appearance {+edge}}");
    normalization("none");
    bboxDraw(false);
  };

  void prologue() {
    *conn << "(geometry " << name << " {";
  }

  void epilogue() {
    *conn << "})";
  }
};

// }}}
// {{{ class gvCam

class gvCam : public gvObj {
public:
  gvCam(const char *name, ostream* s) : gvObj(name, s) {};

  void backcolor(const colour& col) { // background colour
    send("backcolor", col);
  }

  void camDraw(bool b) { // whether or not cameras should be drawn
    send("camera-draw", b);
  }

  //  void camProp() { // {{{ 
  //     `(camera-prop { geometry object }   [projective])'
  //  	  Specify the object to be shown when drawing other cameras.
  //  	  By default, this object is drawn with its origin at the camera,
  //  	  and with the camera looking toward the object's -Z axis.
  //  	  With the `projective' keyword, the camera's viewing projection is
  //  	  also applied to the object; this places the object's Z=-1 and Z=+1 at
  //  	  near and far clipping planes, with the viewing area -1<={X,Y}<=+1.
  //  	  Example:  (camera-prop { < cube } projective)// }}}
  // }
    
  void dither(bool b) { // turn dithering on or off
    send("dither", b);
  }

  void draw() { // Draw the view, if necessary.
    send("draw");
  }

  void freeze() { // {{{ 
    // Freeze CAM-ID; drawing in this camera's window is turned off
    // until it is explicitly redrawn with `(redraw CAM-ID)', after
    // which time drawing resumes as normal.// }}}
    send("freeze");
  }
  
  void hmodel(const char* type) { // {{{ 
    // Set the model used to display geometry in
    // this camera; see also `space'.// }}}
    send("hmodel", type);
  }

  void hsphere(bool b){ // {{{ 
    // Say whether to draw a unit sphere: the sphere at infinity in
    // hyperbolic space, and a reference sphere in Euclidean and spherical
    // spaces.// }}}
    send("hsphere-draw", b);
  }

  void look(const gvGeom& g) { // {{{ 
    // Rotates the named camera to point toward the center of the 
    // bounding box of the named object (or the origin in hyperbolic or 
    // spherical space).  In Euclidean space, moves the camera 
    // forward or backward until the object appears as large 
    // as possible while still being entirely visible.
// }}}
    sendPre("look", g.name);
  }

  void lookToward(const gvGeom& g) { // {{{ 
    // Rotates the named camera to point toward the origin of the
    // object's coordinate system, or the center of the object's
    // bounding box (in non-Euclidean space, the origin will be used 
    // automatically).  Default objectID is the world, default camera
    // is targetcam, default location to point towards is the center
    // of the bounding box.// }}}
    sendPre("look-toward", g.name);
  }
  
  void redraw() { // Redraw the view in CAM-ID
    send("redraw");
  }
  
  void reset() {
    send("camera-reset");
  }

  void set(const char* camspec) { // {{{ 
    // Specify data for object; camspec is a string giving an OOGL
    // camera specification.// }}}
    send("camera", camspec);
  }

  void psSnapshot(const char* filename) { // {{{ 
    // Dumps a Postscript picture to filename representing 
    // the view from the camera window; hidden-surface removal might be incorrect.// }}}
    send("snapshot", filename, "ps");
  }

  void zoom(float f) { // {{{ 
    // Zoom CAM-ID, multiplying its field of view by FACTOR.
    // FACTOR should be a positive number.// }}}
    send("zoom", f);
  }
};


// }}}
// {{{ gvObj: global externs

extern gvGeom axes;

// }}}

H_END(gvObj);
C_BEGIN(gvObj);

#include "exception.h"

// {{{ colour declarations

colour colour::Colour[8];
const colour& colour::Red    = colour::Colour[0] = colour(255, 0,   0  );
const colour& colour::Blue   = colour::Colour[1] = colour(0,   0,   255);
const colour& colour::Green  = colour::Colour[2] = colour(0,   255, 0  );
const colour& colour::Yellow = colour::Colour[3] = colour(255, 255, 0  );
const colour& colour::Cyan   = colour::Colour[4] = colour(0,   255, 255);
const colour& colour::Purple = colour::Colour[5] = colour(255, 255, 0  );
const colour& colour::Black  = colour::Colour[6] = colour(0,   0,   0  );
const colour& colour::White  = colour::Colour[7] = colour(255, 255, 255);

// }}}

C_END(gvObj);

// {{{ Local Variables

// Local Variables:
// mode: c++
// folded-file: t
// End:

// }}}
