WARNING;
// Copyright (C) 2000/3 Paul Martin and Dave Woodcock. All rights reserved.
// $Id: urlist.m4,v 1.5 2000/11/13 06:16:49 dave Exp $
H_BEGIN(urlist);

#include <iostream.h>
#ifdef EXTRA_SAFE
#  include "exception.h"
#endif

// {{{ class urlist

template <class X>
class urlist {
private:
  unsigned int num;				  // number of entries
  X* entry;				          // array of entries
#ifdef EXTRA_SAFE
  void verifyRange(unsigned int) const;
#endif
		      
public:			   
  //  static const X zero(0);                         // the zero element
  // PPM:
  static const X zero;                         // the zero element
  urlist(unsigned int = 0);                       // allocates spaces, no intialisation
  urlist(unsigned int i, const X& x);             // initialise i entries to x
  urlist(const urlist<X>&);

  // suburlist of v starting at i of len j
  urlist(const urlist<X>& v, unsigned int i, unsigned int j);
  ~urlist() {if (num) delete [] entry;}

  unsigned int len() const {return num;}
  X& operator[] (unsigned int);			  // indexing
  const X& operator[] (unsigned int) const;	  // indexing for const urUrList
 
  urlist<X>& operator= (const urlist<X>&); 
  bool operator== (const urlist<X>&) const; 
  bool operator!= (const urlist<X>&l) const {return !(*this == l);}
  
  void setLen(unsigned int i);			  // len = i, entries undefined
  void checkLen(unsigned int i);		  // len >= i, entries undefined
  void clearLen(unsigned int i);                  // len >= i, entries zero
  unsigned int nonzeroCount(unsigned int i) const;// number of nonzero entries among first i
  
  ostream& print(ostream&) const;
};
// PPM:
template<class X> const X urlist<X>::zero(0);	//suggested by dw may 2003.

// }}}
// {{{ inline functions

// {{{ urlist<X>::operator[]

template<class X>
inline X& urlist<X>::operator[] (unsigned int i) {
#ifdef EXTRA_SAFE
  verifyRange(i);
#endif
  return entry[i];
}

template<class X>
inline const X& urlist<X>::operator[] (unsigned int i) const {
#ifdef EXTRA_SAFE
  verifyRange(i);
#endif
  return entry[i];
}

// }}}
// {{{ urlist<X>::urlist()

// {{{ (int)

template<class X>
inline urlist<X>::urlist(unsigned int i)
  : num(i)
{
  // doesn't initialise the space
  if (num) entry = new X [num];
}

// }}}
// {{{ (int, const X&)

template<class X>
inline urlist<X>::urlist(unsigned int i, const X& x)
  : num(i)
{
  // doesn't initialise the space
  if (num) entry = new X [num];
  for (unsigned int j = 0; j < num; ++j)
    entry[j] = x;
}

// }}}
// {{{ (const urlist<X> &)

template<class X>
inline urlist<X>::urlist(const urlist<X>& v)
  : num(v.num)
{
  if (num) {
    entry = new X [num];
    for (unsigned int i = 0; i < num; ++i)
      entry[i] = v.entry[i];
  }
}

// }}}
// {{{ (const urlist<X>&, int, int)

template<class X>
inline urlist<X>::urlist(const urlist<X>& v, unsigned int a, unsigned int b)
  : num(b)
{

#ifdef EXTRA_SAFE
  if (a + b > v.num)
    RANGE_ERROR("urlist::urlist()");
#endif

  if (num) {
    entry = new X [num];
    for (unsigned int i = 0; i < num; ++i)
      entry[i] = v.entry[a++];
  }
}

// }}}

// }}}
// {{{ operator<< ()

template<class X>
inline ostream& operator<< (ostream& s, const urlist<X>& v) {
  return v.print(s);
}

// }}}
// {{{ urlist<X>::checkLen()

template<class X>
inline void urlist<X>::checkLen(unsigned int l) {
  if (num < l) {
    if (num) delete [] entry;
    num = (unsigned int)(1.5 * num);
    if (num < l) num = l;
    if (num) entry = new X [num];
  }
}

// }}}
// {{{ urlist<X>::clearLen()

template<class X>
inline void urlist<X>::clearLen(unsigned int l) {
  checkLen(l);
  while (l--) entry[l] = zero;
}

// }}}

// }}}
// {{{ non-inline functions

// {{{ urlist<X>::verifyRange()

#ifdef EXTRA_SAFE
template<class X>
void urlist<X>::verifyRange(unsigned int i) const {
  if (i >= num)
    RANGE_ERROR("urlist::verifyRange()");
}
#endif

// }}}
// {{{ urlist<X>::operator=()

template<class X>
urlist<X>& urlist<X>::operator= (const urlist<X>& v) {
  // Harmless if passed itself
  if (num != v.num) {
    if (num) delete [] entry;
    if ((num = v.num)) entry = new X [num];
  }
  for (unsigned int i = 0; i < num; ++i)
    entry[i] = v.entry[i];
  return *this;
}

// }}}
// {{{ urlist<X>::operator==();

template<class X>
bool urlist<X>::operator==(const urlist<X>& a) const {
  if (num != a.num) return false;
  for (unsigned int i = 0; i < num; ++i)
    if (entry[i] != a.entry[i])
      return false;
  return true;
}

// }}}
// {{{ urlist<X>::setLen()

template<class X>
void urlist<X>::setLen(unsigned int i) {
  if (num != i) {
    if (num) delete [] entry;
    if ((num = i)) entry = new X [num];
  }
}

// }}}
// {{{ urlist<X>::print()

template<class X>
ostream& urlist<X>::print(ostream& s) const {
  s << "[";
  for (unsigned int i = 0; i < num; ++i)
    if (i == 0)
      s << entry[0];
    else
      s << "," << entry[i];
  s << "]";
  return s;
}

// }}}
// {{{ urlist<X>::nonzeroCount()

template<class X>
unsigned int urlist<X>::nonzeroCount(unsigned int l) const {

#ifdef EXTRA_SAFE
  if (l > num)
    RANGE_ERROR("urlist::nonzeroCount()");
#endif

  unsigned int k = 0;
  for (unsigned int i = 0; i < l ; ++i)
    if (entry[i] != zero) ++k;
  return k;
}

// }}}

// }}}

H_END(urlist);

// {{{ Local Variables

// Local Variables:
// mode: c++
// folded-file: t
// End:

// }}}
