WARNING;
// Copyright (C) 2000 Paul Martin and Dave Woodcock. All rights reserved.
// $Id: main.m4,v 1.13 2000/11/13 06:16:58 dave Exp $
// #define VERBOSE_TILTING_WALK to get diagnostics in tiltingWalk
H_BEGIN(main);
// This file deliberately left blank
H_END(main);
C_BEGIN(main);

#include <unistd.h>
#include <iostream.h>
#include <stdlib.h>
#include "gvObj.h"
#include "rootSystem.h"
#include "intpolhash.h"
#include "alcove.h"
#include "alcovehash.h"
#include "tiltChar.h"
#include "misc.h"
// calc.cc is built by bison,
// hence it's easier to have an ordinary header file

// DJW pig
// #include "parser.h"
// #include "eval.h"

alcovehash alchash(15,5000);
intpolhash polhash(15,5000);
rootSystem R("A3");

// {{{ global functions related to geomview

void operator<< (gvGeom& obj, const alcove& a) {
  obj.prologue();
  // gvout << "INST transforms {:weylGroup} geom ";
  a.gvPrint(gvout);
  obj.epilogue();
}

void operator<< (gvGeom& obj, const character& c) {
  obj.prologue();
  c.gvPrintExteriorWalls(gvout);
  obj.epilogue();
}

// }}}
// {{{ random alcove walk

void randomAlcoveWalk() {
  gvGeom rat("rat");
  affineWeyl w = R.s[0];

  while (true) {
    rat << *alchash.addr(w);
    gvout.flush();
    w = R.s[rand() % (R.rank + 1)] * w;
    sleep(1);
  }
}

// }}}
// {{{ tiltingWalk

void tiltingWalk(const intvec& transient, const intvec& periodic) {
  // Sample values:
  // A3: 0123
  // B3: 0213213
  // B2: 0212

  gvGeom tilt("tilt");
  affineWeyl w(&R, 0, R.origin); // identity affine Weyl element
  bool inTransient = (transient.len() > 0);
  bool isPeriodic  = (periodic.len() > 0);
  unsigned int i = 0;
  unsigned int count = 0;

  while (count < 50) {
#ifdef VERBOSE_TILTING_WALK
    cout << "w = " << w << "\n"
	 << "a = " << *alchash.addr(w) << " (" << alchash.addr(w)->w() << ")\n";
#endif
    ++count;
    const character* foo = tiltChar(alchash.addr(w));
    tilt << *foo;
    gvout.flush();

    sleep(1);

    if (inTransient) {
      w = R.s[transient[i++]] * w;
      if (i >= transient.len()) {
	inTransient = false;
	i = 0;
      }

    } else if (isPeriodic) {
      w = R.s[periodic[i++]] * w;
      if (i >= periodic.len())
	i = 0;

    } else {
      break;
    }
  }
}

// void* PTtiltingWalk(void* transient, void* periodic) {
//   tiltingWalk(*(intvec*)(transient), *(intvec*)(periodic));
//   lastType = VT_NULL;
//   return 0;
// }

// }}}
// {{{ PTaxes (axis printing stubb)

void* PTaxes(void* mag) {
  int magnification = *((int *)(mag));
  static unsigned int i, j;
  axes.prologue();
  gvout << "VECT "
	<< R.rank << " " << (2 * R.rank) << " 0\n"; // num lines, vertices, colours
  for (i = R.rank; i--;)
    gvout << "2 "; // num vertices in each line
  gvout << "\n";
  for (i = R.rank; i--;)
    gvout << "0 "; // num colours in each line
  gvout << "\n";
  intvec& origin = R.fundamentalVertex[0];
  for (i = 1; i <= R.rank; ++i) {
    intvec pt(R.fundamentalVertex[i] * magnification);
    for (j = 0; j < R.rank; ++j)
      gvout << origin[j] << " ";
    for (; j < 3; ++j)
      gvout << "0 ";
    gvout << "\n";
    for (j = 0; j < R.rank; ++j)
      gvout << pt[j] << " ";
    for (; j < 3; ++j)
      gvout << "0 ";
    gvout << "\n";
  }
  axes.epilogue();
  gvout.flush();
  lastType = VT_NULL;
  return 0;
}

// }}}
// {{{ gvopen and gvclose stubs

void * PTgvopen() {
  gvout.open(4);
  world.xform(R.onTransform / 200);
  lastType = VT_NULL;
  return 0;
}

void * PTgvclose() {
  gvout.close();
  lastType = VT_NULL;
  return 0;
}

// }}}
// {{{ main()

int main() {
  // DJW pig
  // parseInit();

  // gvout << "(read geometry {define weylGroup { ";
  // R.printWeylGroup(gvout);
  // gvout << "}})\n";

  // DJW pig
  // addSystem("R", R);
  // addSystem("tiltingWalk", &PTtiltingWalk, VT_INTVEC, VT_INTVEC);
  // addSystem("gvopen", &PTgvopen);
  // addSystem("gvclose", &PTgvclose);
  // addSystem("axes", &PTaxes, VT_INT);
  // 
  // while (!parse());
}

// }}}

C_END(main);

// {{{ Local Variables

// Local Variables:
// mode: c++
// folded-file: t
// End:

// }}}
